@@ -1,6 +1,7 @@
 from io import BytesIO
 from pathlib import Path
 
+import dkist_fits_specifications
 import numpy as np
 import pytest
 from astropy.io import fits
@@ -12,7 +13,100 @@ from dkist_header_validator.exceptions import ValidationException
 
 
 @pytest.fixture(scope="module")
-def valid_spec_122_headers(tmpdir_factory):
+def valid_spec_122_headers_validate(tmpdir_factory):
+    """
+    Create a dict of valid spec 122 headers to be used in successful
+    header tests below.
+    """
+    valid_spec_122_dict = {
+        "NAXIS": 3,
+        "BITPIX": 16,
+        "NAXIS1": 1,
+        "NAXIS2": 1,
+        "NAXIS3": 1,
+        "INSTRUME": "VBI-BLUE",
+        "WAVELNTH": 430.0,
+        "DATE-BGN": "2017-05-29T12:00:13.345",
+        "DATE-END": "2017-05-30T20:00:13.345",
+        "CHECKSUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIUF",
+        "DATE-OBS": "2017-05-30T00:46:13.952",
+        "ID___002": "YVPS4YRBSXUT9Z17Z4HRH3VIH7T6KO",
+        "ID___008": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
+        "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
+        "ID___013": "2KJBWEFB4OUUBSFUIB5JKBSDF8JBSK",
+        "DKIST003": "OSZ4FBHWKXRWQGOVG9BJNUWNG5795B",
+        "DKIST004": "Observation",
+        "WCSAXES": 3,
+        "WCSNAME": "Helioprojective Cartesian",
+        "CRPIX1": 13.4,
+        "CRPIX2": 14.6,
+        "CRPIX3": 15.6,
+        "CRVAL1": 16.7,
+        "CRVAL2": 18.5,
+        "CRVAL3": 18.6,
+        "CDELT1": 20.4,
+        "CDELT2": 67.8,
+        "CDELT3": 78.8,
+        "CUNIT1": "deg",
+        "CUNIT2": "deg",
+        "CUNIT3": "deg",
+        "CTYPE1": "x",
+        "CTYPE2": "y",
+        "CTYPE3": "z",
+        "PC1_1": 13.5,
+        "PC1_2": 13.5,
+        "PC2_1": 13.5,
+        "PC2_2": 13.5,
+        "PC1_3": 13.5,
+        "PC3_2": 13.5,
+        "PC2_3": 13.5,
+        "PC3_1": 13.5,
+        "PC3_3": 13.5,
+        "BUNIT": "ct",
+        "DATE": "2017-05-30T00:46:13.952",
+        "ORIGIN": "4L6XY2SM39CNQTOO4L04Y3RV0H2MTW",
+        "TELESCOP": "DKIST",
+        "OBSERVAT": "NSO",
+        "NETWORK": "ABCD",
+        "OBJECT": "SUNSPOT N62",
+        "DATASUM": "E5O2YIVIP04EOEL59NGM",
+        "HISTORY": "Old History",
+        "COMMENT": "A comment",
+    }
+
+    temp_dir = tmpdir_factory.mktemp("valid spec_122_headers_temp")
+    file_name = temp_dir.join("tmp_fits_file.fits")
+    temp_array = np.ones((1, 1, 1), dtype=np.int16)
+    valid_hdu = fits.PrimaryHDU(temp_array)
+    # Use the valid_spec_122_dict from above to overwrite the default header
+    for (key, value) in valid_spec_122_dict.items():
+        valid_hdu.header[key] = value
+    valid_hdu_list = fits.HDUList([valid_hdu])
+    valid_hdu_list.writeto(str(file_name))
+
+    yield {
+        "valid_dkist_hdr.fits": Path(file_name),
+        "valid_spec_122_dict": valid_spec_122_dict,
+        "valid_HDUList": valid_hdu_list,
+        "valid header": valid_hdu.header,
+    }
+
+
+@pytest.fixture(
+    scope="function",
+    params=[
+        "valid_dkist_hdr.fits",
+        "valid_spec_122_dict",
+        "valid_HDUList",
+        "valid header",
+    ],
+)
+def valid_spec_122_header_validate(request, valid_spec_122_headers_validate):
+    yield valid_spec_122_headers_validate[request.param]
+
+
+@pytest.fixture(scope="module")
+def valid_spec_122_headers_validate_and_translate(tmpdir_factory):
     """
     Create a dict of valid spec 122 headers to be used in successful
     header tests below.
@@ -98,11 +192,13 @@ def valid_spec_122_headers(tmpdir_factory):
         "valid header",
     ],
 )
-def valid_spec_122_header(request, valid_spec_122_headers):
-    yield valid_spec_122_headers[request.param]
+def valid_spec_122_header_validate_and_translate(
+    request, valid_spec_122_headers_validate_and_translate
+):
+    yield valid_spec_122_headers_validate_and_translate[request.param]
 
 
-def test_spec122_valid(valid_spec_122_header):
+def test_spec122_valid(valid_spec_122_header_validate):
     """
     Validates a fits header against the SPEC-122 schema
     Given: A valid SPEC-122 fits header
@@ -110,10 +206,10 @@ def test_spec122_valid(valid_spec_122_header):
     Then: return validated HDUList and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate(valid_spec_122_header, extra=False)
+    spec122_validator.validate(valid_spec_122_header_validate, extra=False)
 
 
-def test_spec122_valid_return_dictionary(valid_spec_122_header):
+def test_spec122_valid_return_dictionary(valid_spec_122_header_validate):
     """
     Validates a fits header against the SPEC-122 schema
     Given: A valid SPEC-122 fits header
@@ -121,7 +217,26 @@ def test_spec122_valid_return_dictionary(valid_spec_122_header):
     Then: return validated dictionary and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate(valid_spec_122_header, return_type=dict, extra=False)
+    spec122_validator.validate(valid_spec_122_header_validate, return_type=dict, extra=False)
+
+
+def test_spec122_WAVELNTH_not_renamed(valid_spec_122_header_validate_and_translate):
+    """
+    Given: A valid schema definition
+    When: Validating and translating the schema
+    Then: Check that the rename of keywords is correctly being renamed
+    """
+    default_values = [-999.9, -999, "default"]
+
+    spec214schema = list(dkist_fits_specifications.spec214.load_spec214().values())
+    headers = spec122_validator.validate_and_translate(
+        valid_spec_122_header_validate_and_translate, return_type=dict, extra=False
+    )
+
+    for section in spec214schema:
+        for key, schema in section.items():
+            if schema.get("rename") and key in headers and headers[key] == any(default_values):
+                raise Spec122ValidationException
 
 
 @pytest.fixture(
@@ -130,8 +245,8 @@ def test_spec122_valid_return_dictionary(valid_spec_122_header):
         "valid_dkist_hdr.fits",
     ],
 )
-def valid_spec_122_file(request, valid_spec_122_headers):
-    yield valid_spec_122_headers[request.param]
+def valid_spec_122_file(request, valid_spec_122_headers_validate):
+    yield valid_spec_122_headers_validate[request.param]
 
 
 def test_spec122_valid_return_BytesIO(valid_spec_122_file):
@@ -153,8 +268,8 @@ def test_spec122_valid_return_BytesIO(valid_spec_122_file):
         "valid header",
     ],
 )
-def valid_spec_122_header_fail_bytesio_and_file(request, valid_spec_122_headers):
-    yield valid_spec_122_headers[request.param]
+def valid_spec_122_header_fail_bytesio_and_file(request, valid_spec_122_headers_validate):
+    yield valid_spec_122_headers_validate[request.param]
 
 
 def test_spec122_valid_return_BytesIO_failure(valid_spec_122_header_fail_bytesio_and_file):
@@ -171,7 +286,7 @@ def test_spec122_valid_return_BytesIO_failure(valid_spec_122_header_fail_bytesio
         )
 
 
-def test_spec122_valid_return_fits_header(valid_spec_122_header):
+def test_spec122_valid_return_fits_header(valid_spec_122_header_validate):
     """
     Validates a fits header against the SPEC-122 schema
     Given: A valid SPEC-122 fits header
@@ -179,7 +294,9 @@ def test_spec122_valid_return_fits_header(valid_spec_122_header):
     Then: return validated fits.header.Header object and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate(valid_spec_122_header, return_type=fits.header.Header, extra=False)
+    spec122_validator.validate(
+        valid_spec_122_header_validate, return_type=fits.header.Header, extra=False
+    )
 
 
 def test_spec122_valid_return_file_failure(valid_spec_122_header_fail_bytesio_and_file):
@@ -207,7 +324,7 @@ def test_spec122_valid_return_file(valid_spec_122_file):
     spec122_validator.validate(valid_spec_122_file, return_type=Path, extra=False)
 
 
-def test_validate_and_translate_spec122(valid_spec_122_header):
+def test_validate_and_translate_spec122(valid_spec_122_header_validate_and_translate):
     """
     Validates and tries to translate a fits header against the SPEC-122 schema
     Given: A valid SPEC-122 fits header
@@ -215,10 +332,12 @@ def test_validate_and_translate_spec122(valid_spec_122_header):
     Then: return translated HDUList and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate_and_translate(valid_spec_122_header)
+    spec122_validator.validate_and_translate(valid_spec_122_header_validate_and_translate)
 
 
-def test_validate_and_translate_spec122_return_dictionary(valid_spec_122_header):
+def test_validate_and_translate_spec122_return_dictionary(
+    valid_spec_122_header_validate_and_translate,
+):
     """
     Validates and tries to translate a fits header against the SPEC-122 schema
     Given: A valid SPEC-122 fits header
@@ -226,7 +345,9 @@ def test_validate_and_translate_spec122_return_dictionary(valid_spec_122_header)
     Then: return translated dictionary and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate_and_translate(valid_spec_122_header, return_type=dict)
+    spec122_validator.validate_and_translate(
+        valid_spec_122_header_validate_and_translate, return_type=dict
+    )
 
 
 def test_validate_and_translate_spec122_return_BytesIO(valid_spec_122_file):
@@ -240,7 +361,9 @@ def test_validate_and_translate_spec122_return_BytesIO(valid_spec_122_file):
     spec122_validator.validate_and_translate(valid_spec_122_file, return_type=BytesIO)
 
 
-def test_validate_and_translate_spec122_return_fits_header(valid_spec_122_header):
+def test_validate_and_translate_spec122_return_fits_header(
+    valid_spec_122_header_validate_and_translate,
+):
     """
     Validates and tries to translate a fits header against the SPEC-122 schema
     Given: A valid SPEC-122 fits header
@@ -248,7 +371,9 @@ def test_validate_and_translate_spec122_return_fits_header(valid_spec_122_header
     Then: return translated fits.header.Header object and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate_and_translate(valid_spec_122_header, return_type=fits.header.Header)
+    spec122_validator.validate_and_translate(
+        valid_spec_122_header_validate_and_translate, return_type=fits.header.Header
+    )
 
 
 def test_validate_and_translate_spec122_return_file(valid_spec_122_file):
@@ -379,97 +504,7 @@ def test_spec122_valid_extrakeysfalse(valid_spec_122_header_extrakeys):
         spec122_validator.validate_and_translate(valid_spec_122_header_extrakeys, extra=False)
 
 
-@pytest.fixture(scope="module")
-def valid_spec_122_headers_translate(tmpdir_factory):
-    """
-    Create a dict of valid spec 122 headers to be used in successful
-    header tests below.
-    """
-    valid_spec_122_dict_translate = {
-        "NAXIS": 3,
-        "BITPIX": 16,
-        "NAXIS1": 1,
-        "NAXIS2": 1,
-        "NAXIS3": 1,
-        "WCSAXES": 1,
-        "WCSNAME": "VNSNETLCAJ33XKUOFDGD",
-        "INSTRUME": "VBI-BLUE",
-        "WAVELNTH": 430.0,
-        "CHECKSUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIUF",
-        "DATE-OBS": "2017-05-30T00:46:13.952",
-        "ID___002": "YVPS4YRBSXUT9Z17Z4HRH3VIH7T6KO",
-        "ID___008": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
-        "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
-        "ID___013": "2KJBWEFB4OUUBSFUIB5JKBSDF8JBSK",
-        "DKIST003": "OSZ4FBHWKXRWQGOVG9BJNUWNG5795B",
-        "DKIST004": "Observation",
-        "CRPIX1": 13.4,
-        "CRPIX2": 14.6,
-        "CRPIX3": 15.6,
-        "CRVAL1": 16.7,
-        "CRVAL2": 18.5,
-        "CRVAL3": 18.6,
-        "CDELT1": 20.4,
-        "CDELT2": 67.8,
-        "CDELT3": 78.8,
-        "CUNIT1": "deg",
-        "CUNIT2": "deg",
-        "CUNIT3": "deg",
-        "CTYPE1": "x",
-        "CTYPE2": "y",
-        "CTYPE3": "z",
-        "PC1_1": 13.5,
-        "PC1_2": 13.5,
-        "PC2_1": 13.5,
-        "PC2_2": 13.5,
-        "PC1_3": 13.5,
-        "PC3_2": 13.5,
-        "PC2_3": 13.5,
-        "PC3_1": 13.5,
-        "PC3_3": 13.5,
-        "BUNIT": "ct",
-        "DATE": "2017-05-30T00:46:13.952",
-        "DATE-END": "2017-05-30T00:46:13.952",
-        "ORIGIN": "4L6XY2SM39CNQTOO4L04Y3RV0H2MTW",
-        "TELESCOP": "DKIST",
-        "OBSERVAT": "NSO",
-        "NETWORK": "ABCD",
-        "OBJECT": "SUNSPOT N62",
-        "DATASUM": "E5O2YIVIP04EOEL59NGM",
-    }
-
-    temp_dir = tmpdir_factory.mktemp("valid spec_122_headers_translate_temp")
-    file_name = temp_dir.join("tmp_fits_file.fits")
-    temp_array = np.ones((1, 1, 1), dtype=np.int16)
-    valid_hdu_translate = fits.PrimaryHDU(temp_array)
-    # Use the valid_spec_122_dict_translate from above to overwrite the default header
-    for (key, value) in valid_spec_122_dict_translate.items():
-        valid_hdu_translate.header[key] = value
-    valid_hdu_list_translate = fits.HDUList([valid_hdu_translate])
-    valid_hdu_list_translate.writeto(str(file_name))
-
-    yield {
-        "valid_dkist_hdr_translate.fits": Path(file_name),
-        "valid_spec_122_dict_translate": valid_spec_122_dict_translate,
-        "valid_HDUList_translate": valid_hdu_list_translate,
-        "valid header translate": valid_hdu_translate.header,
-    }
-
-
-@pytest.fixture(
-    scope="function",
-    params=[
-        "valid_dkist_hdr_translate.fits",
-        "valid_spec_122_dict_translate",
-        "valid_HDUList_translate",
-        "valid header translate",
-    ],
-)
-def valid_spec_122_header_translate(request, valid_spec_122_headers_translate):
-    yield valid_spec_122_headers_translate[request.param]
-
-
-def test_spec122_valid_translate(valid_spec_122_header_translate):
+def test_spec122_valid_translate(valid_spec_122_header_validate_and_translate):
     """
     Validates a fits header against the SPEC-0122 schema
     then translates SPEC-0122 fits headers to SPEC-122 headers
@@ -478,7 +513,7 @@ def test_spec122_valid_translate(valid_spec_122_header_translate):
     Then: return validated HDUList and do not raise an exception
     """
     # raises exception on failure
-    spec122_validator.validate_and_translate(valid_spec_122_header_translate)
+    spec122_validator.validate_and_translate(valid_spec_122_header_validate_and_translate)
 
 
 @pytest.fixture(scope="module")
@@ -696,6 +731,7 @@ def max_headers(tmpdir_factory):
         "PC2_3": 13.5,
         "PC3_1": 13.5,
         "PC3_3": 13.5,
+        "HISTORY": "Old History",
     }
 
     temp_dir = tmpdir_factory.mktemp("max_headers_temp")
@@ -737,9 +773,11 @@ def valid_compressed_spec_122_headers(tmpdir_factory):
         "BSCALE": 1.0,
         "INSTRUME": "VBI-BLUE",
         "WAVELNTH": 430.0,
+        "DATE-END": "2017-05-30T00:46:13.718",
         "CHECKSUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIUF",
         "DATASUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIZY",
         "DATE-OBS": "2017-05-30T00:46:13.952",
+        "DATE-BGN": "2017-05-30T00:46:13.952",
         "ID___002": "YVPS4YRBSXUT9Z17Z4HRH3VIH7T6KO",
         "ID___008": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
         "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
@@ -784,12 +822,13 @@ def valid_compressed_spec_122_headers(tmpdir_factory):
     temp_dir = tmpdir_factory.mktemp("valid comp_122_headers_temp")
     file_name = temp_dir.join("tmp__comp_fits_file.fits")
     temp_array = np.ones((1, 1, 1), dtype=np.int16)
-    valid_comp_hdu = fits.PrimaryHDU(temp_array)
+    primary_hdu = fits.PrimaryHDU()
+    valid_comp_hdu = fits.CompImageHDU(temp_array)
     # Use the valid_spec_122_dict from above to overwrite the default header
     for (key, value) in valid_comp_122_dict.items():
         valid_comp_hdu.header[key] = value
-    valid_comp_hdu_list = fits.HDUList([valid_comp_hdu])
-    valid_comp_hdu_list.writeto(str(file_name))
+    valid_comp_hdu_list = fits.HDUList([primary_hdu, valid_comp_hdu])
+    valid_comp_hdu_list.writeto(str(file_name), checksum=True)
 
     yield {
         "valid_compressed_hdr.fits.fz": Path(file_name),
@@ -814,3 +853,355 @@ def test_compressed_spec122_valid(valid_compressed_spec_122_header):
     Then: return valid HDUList and do not raise an exception
     """
     spec122_validator.validate(valid_compressed_spec_122_header)
+
+
+def test_compressed_spec122_validate_and_translate(valid_compressed_spec_122_header):
+    """
+    Validates and translates a compressed spec122 compliant file
+    Given: A valid compressed SPEC-0122 file
+    When: Validating headers
+    Then: return valid HDUList and do not raise an exception
+    """
+    spec122_validator.validate_and_translate(valid_compressed_spec_122_header)
+
+
+@pytest.fixture(scope="module")
+def valid_visp_122_headers(tmpdir_factory):
+    """
+    Create a dict of valid visp spec 122 headers to be used in successful
+    header tests below.
+    """
+    valid_visp_122_dict = {
+        "NAXIS": 3,
+        "BITPIX": 16,
+        "NAXIS1": 1,
+        "NAXIS2": 1,
+        "NAXIS3": 1,
+        "INSTRUME": "VBI-BLUE",
+        "WAVELNTH": 430.0,
+        "DATE-BGN": "2017-05-29T12:00:13.345",
+        "DATE-END": "2017-05-30T20:00:13.345",
+        "CHECKSUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIUF",
+        "DATE-OBS": "2017-05-30T00:46:13.952",
+        "VISP_001": 3,
+        "VISP_002": 32.0,
+        "VISP_003": 45.6,
+        "VISP_004": "string",
+        "VISP_005": 31.9,
+        "ID___002": "YVPS4YRBSXUT9Z17Z4HRH3VIH7T6KO",
+        "ID___008": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
+        "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
+        "ID___013": "2KJBWEFB4OUUBSFUIB5JKBSDF8JBSK",
+        "DKIST003": "OSZ4FBHWKXRWQGOVG9BJNUWNG5795B",
+        "DKIST004": "Observation",
+        "WCSAXES": 3,
+        "WCSNAME": "Helioprojective Cartesian",
+        "CRPIX1": 13.4,
+        "CRPIX2": 14.6,
+        "CRPIX3": 15.6,
+        "CRVAL1": 16.7,
+        "CRVAL2": 18.5,
+        "CRVAL3": 18.6,
+        "CDELT1": 20.4,
+        "CDELT2": 67.8,
+        "CDELT3": 78.8,
+        "CUNIT1": "deg",
+        "CUNIT2": "deg",
+        "CUNIT3": "deg",
+        "CTYPE1": "x",
+        "CTYPE2": "y",
+        "CTYPE3": "z",
+        "PC1_1": 13.5,
+        "PC1_2": 13.5,
+        "PC2_1": 13.5,
+        "PC2_2": 13.5,
+        "PC1_3": 13.5,
+        "PC3_2": 13.5,
+        "PC2_3": 13.5,
+        "PC3_1": 13.5,
+        "PC3_3": 13.5,
+        "BUNIT": "ct",
+        "DATE": "2017-05-30T00:46:13.952",
+        "ORIGIN": "4L6XY2SM39CNQTOO4L04Y3RV0H2MTW",
+        "TELESCOP": "DKIST",
+        "OBSERVAT": "NSO",
+        "NETWORK": "ABCD",
+        "OBJECT": "SUNSPOT N62",
+        "DATASUM": "E5O2YIVIP04EOEL59NGM",
+    }
+
+    temp_dir = tmpdir_factory.mktemp("valid visp_122_headers_temp")
+    file_name = temp_dir.join("tmp_fits_file.fits")
+    temp_array = np.ones((1, 1, 1), dtype=np.int16)
+    valid_hdu = fits.PrimaryHDU(temp_array)
+    # Use the valid_visp_122_dict from above to overwrite the default header
+    for (key, value) in valid_visp_122_dict.items():
+        valid_hdu.header[key] = value
+    valid_hdu_list = fits.HDUList([valid_hdu])
+    valid_hdu_list.writeto(str(file_name))
+
+    yield {
+        "valid_visp_hdr.fits": Path(file_name),
+        "valid_visp_122_dict": valid_visp_122_dict,
+        "valid_visp_HDUList": valid_hdu_list,
+        "valid visp header": valid_hdu.header,
+    }
+
+
+@pytest.fixture(
+    scope="function",
+    params=[
+        "valid_visp_hdr.fits",
+        "valid_visp_122_dict",
+        "valid_visp_HDUList",
+        "valid visp header",
+    ],
+)
+def valid_visp_122_header(request, valid_visp_122_headers):
+    yield valid_visp_122_headers[request.param]
+
+
+def test_visp_spec122_valid(valid_visp_122_header):
+    """
+    Validates a visp fits header against the SPEC-122 schema
+    Given: A valid visp SPEC-122 fits header
+    When: Validating headers
+    Then: return validated HDUList and do not raise an exception
+    """
+    # raises exception on failure
+    spec122_validator.validate(valid_visp_122_header, extra=False)
+
+
+def test_visp_spec122_valiate_and_translate(valid_visp_122_header):
+    """
+    Validates a visp fits header against the SPEC-122 schema
+    Given: A valid visp SPEC-122 fits header
+    When: Validating headers
+    Then: return validated HDUList and do not raise an exception
+    """
+    # raises exception on failure
+    spec122_validator.validate_and_translate(valid_visp_122_header, return_type=dict)
+
+
+@pytest.fixture(scope="module")
+def valid_spec_122_headers_toomanyHDUs(tmpdir_factory):
+    """
+    Create a dict of valid spec 122 headers.
+    """
+    valid_spec_122_dict = {
+        "NAXIS": 3,
+        "BITPIX": 16,
+        "NAXIS1": 1,
+        "NAXIS2": 1,
+        "NAXIS3": 1,
+        "INSTRUME": "VBI-BLUE",
+        "WAVELNTH": 430.0,
+        "DATE-BGN": "2017-05-29T12:00:13.345",
+        "DATE-END": "2017-05-30T20:00:13.345",
+        "CHECKSUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIUF",
+        "DATE-OBS": "2017-05-30T00:46:13.952",
+        "ID___002": "YVPS4YRBSXUT9Z17Z4HRH3VIH7T6KO",
+        "ID___008": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
+        "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
+        "ID___013": "2KJBWEFB4OUUBSFUIB5JKBSDF8JBSK",
+        "DKIST003": "OSZ4FBHWKXRWQGOVG9BJNUWNG5795B",
+        "DKIST004": "Observation",
+        "WCSAXES": 3,
+        "WCSNAME": "Helioprojective Cartesian",
+        "CRPIX1": 13.4,
+        "CRPIX2": 14.6,
+        "CRPIX3": 15.6,
+        "CRVAL1": 16.7,
+        "CRVAL2": 18.5,
+        "CRVAL3": 18.6,
+        "CDELT1": 20.4,
+        "CDELT2": 67.8,
+        "CDELT3": 78.8,
+        "CUNIT1": "deg",
+        "CUNIT2": "deg",
+        "CUNIT3": "deg",
+        "CTYPE1": "x",
+        "CTYPE2": "y",
+        "CTYPE3": "z",
+        "PC1_1": 13.5,
+        "PC1_2": 13.5,
+        "PC2_1": 13.5,
+        "PC2_2": 13.5,
+        "PC1_3": 13.5,
+        "PC3_2": 13.5,
+        "PC2_3": 13.5,
+        "PC3_1": 13.5,
+        "PC3_3": 13.5,
+        "BUNIT": "ct",
+        "DATE": "2017-05-30T00:46:13.952",
+        "ORIGIN": "4L6XY2SM39CNQTOO4L04Y3RV0H2MTW",
+        "TELESCOP": "DKIST",
+        "OBSERVAT": "NSO",
+        "NETWORK": "ABCD",
+        "OBJECT": "SUNSPOT N62",
+        "DATASUM": "E5O2YIVIP04EOEL59NGM",
+    }
+
+    temp_dir = tmpdir_factory.mktemp("valid spec_122_headers_temp")
+    file_name = temp_dir.join("tmp_fits_file.fits")
+    temp_array = np.ones((1, 1, 1), dtype=np.int16)
+    valid_hdu = fits.PrimaryHDU(temp_array)
+    image_hdu1 = fits.ImageHDU(temp_array)
+    image_hdu2 = fits.ImageHDU(temp_array)
+    # Use the valid_spec_122_dict from above to overwrite the default header
+    for (key, value) in valid_spec_122_dict.items():
+        valid_hdu.header[key] = value
+        image_hdu1.header[key] = value
+        image_hdu2.header[key] = value
+    valid_hdu_list = fits.HDUList([valid_hdu, image_hdu1, image_hdu2])
+    valid_hdu_list.writeto(str(file_name))
+
+    yield {
+        "valid_dkist_hdr_toomanyHDUs.fits": Path(file_name),
+        "valid_HDUList_toomanyHDUs": valid_hdu_list,
+    }
+
+
+@pytest.fixture(
+    scope="function",
+    params=[
+        "valid_dkist_hdr_toomanyHDUs.fits",
+        "valid_HDUList_toomanyHDUs",
+    ],
+)
+def valid_spec_122_header_toomanyHDUs(request, valid_spec_122_headers_toomanyHDUs):
+    yield valid_spec_122_headers_toomanyHDUs[request.param]
+
+
+def test_toomanyHDUs_validate_and_translate(valid_spec_122_header_toomanyHDUs):
+    """
+    Validates headers with too many (more than 2) HDUs
+    Given: A valid SPEC-122 file or HDUList with more than two headers
+    When: Validating and translating headers
+    Then: Raise an exception
+    """
+    # raises exception on failure
+    with pytest.raises(ValidationException):
+        spec122_validator.validate_and_translate(valid_spec_122_header_toomanyHDUs)
+
+
+def test_toomanyHDUs_validate(valid_spec_122_header_toomanyHDUs):
+    """
+    Validates headers with too many (more than 2) HDUs
+    Given: A valid SPEC-122 file or HDUList with more than two headers
+    When: Validating headers
+    Then: Raise an exception
+    """
+    # raises exception on failure
+    with pytest.raises(ValidationException):
+        spec122_validator.validate(valid_spec_122_header_toomanyHDUs)
+
+
+@pytest.fixture(scope="module")
+def valid_spec_122_headers_datainsecondHDU(tmpdir_factory):
+    """
+    Create a dict of valid spec 122 headers.
+    """
+    valid_spec_122_dict = {
+        "NAXIS": 3,
+        "BITPIX": 16,
+        "NAXIS1": 1,
+        "NAXIS2": 1,
+        "NAXIS3": 1,
+        "INSTRUME": "VBI-BLUE",
+        "WAVELNTH": 430.0,
+        "DATE-BGN": "2017-05-29T12:00:13.345",
+        "DATE-END": "2017-05-30T20:00:13.345",
+        "CHECKSUM": "POLETJWHTN2PMM1ZPPLPWQ1KBAKIUF",
+        "DATE-OBS": "2017-05-30T00:46:13.952",
+        "ID___002": "YVPS4YRBSXUT9Z17Z4HRH3VIH7T6KO",
+        "ID___008": "JX3O8NXFI6FGTVZ1D7G7U8OVUWDZQL",
+        "ID___012": "1XXPIDR5CEXMZ0SQ8LT3HMF83FW4HJ",
+        "ID___013": "2KJBWEFB4OUUBSFUIB5JKBSDF8JBSK",
+        "DKIST003": "OSZ4FBHWKXRWQGOVG9BJNUWNG5795B",
+        "DKIST004": "Observation",
+        "WCSAXES": 3,
+        "WCSNAME": "Helioprojective Cartesian",
+        "CRPIX1": 13.4,
+        "CRPIX2": 14.6,
+        "CRPIX3": 15.6,
+        "CRVAL1": 16.7,
+        "CRVAL2": 18.5,
+        "CRVAL3": 18.6,
+        "CDELT1": 20.4,
+        "CDELT2": 67.8,
+        "CDELT3": 78.8,
+        "CUNIT1": "deg",
+        "CUNIT2": "deg",
+        "CUNIT3": "deg",
+        "CTYPE1": "x",
+        "CTYPE2": "y",
+        "CTYPE3": "z",
+        "PC1_1": 13.5,
+        "PC1_2": 13.5,
+        "PC2_1": 13.5,
+        "PC2_2": 13.5,
+        "PC1_3": 13.5,
+        "PC3_2": 13.5,
+        "PC2_3": 13.5,
+        "PC3_1": 13.5,
+        "PC3_3": 13.5,
+        "BUNIT": "ct",
+        "DATE": "2017-05-30T00:46:13.952",
+        "ORIGIN": "4L6XY2SM39CNQTOO4L04Y3RV0H2MTW",
+        "TELESCOP": "DKIST",
+        "OBSERVAT": "NSO",
+        "NETWORK": "ABCD",
+        "OBJECT": "SUNSPOT N62",
+        "DATASUM": "E5O2YIVIP04EOEL59NGM",
+    }
+
+    temp_dir = tmpdir_factory.mktemp("valid spec_122_headers_temp")
+    file_name = temp_dir.join("tmp_fits_file.fits")
+    temp_array = np.ones((1, 1, 1), dtype=np.int16)
+    valid_hdu = fits.PrimaryHDU()
+    image_hdu1 = fits.ImageHDU(temp_array)
+    # Use the valid_spec_122_dict from above to overwrite the default header
+    for (key, value) in valid_spec_122_dict.items():
+        valid_hdu.header[key] = value
+        image_hdu1.header[key] = value
+    valid_hdu_list = fits.HDUList([valid_hdu, image_hdu1])
+    valid_hdu_list.writeto(str(file_name))
+
+    yield {
+        "valid_dkist_hdr_datainsecondHDU.fits": Path(file_name),
+    }
+
+
+@pytest.fixture(
+    scope="function",
+    params=[
+        "valid_dkist_hdr_datainsecondHDU.fits",
+    ],
+)
+def valid_spec_122_header_datainsecondHDU(request, valid_spec_122_headers_datainsecondHDU):
+    yield valid_spec_122_headers_datainsecondHDU[request.param]
+
+
+def test_datainsecondHDU_validate_and_translate(valid_spec_122_header_datainsecondHDU):
+    """
+    Validates headers with data stored in second HDU
+    Given: A valid SPEC-122 file or with data stored in second HDU
+    When: Validating and translating headers
+    Then: Raise an exception
+    """
+    # raises exception on failure
+    spec122_validator.validate_and_translate(
+        valid_spec_122_header_datainsecondHDU, return_type=Path
+    )
+
+
+def test_datainsecondHDU_validate(valid_spec_122_header_datainsecondHDU):
+    """
+    Validates headers with data stored in second HDU
+    Given: A valid SPEC-122 file or with data stored in second HDU
+    When: Validating headers
+    Then: Raise an exception
+    """
+    # raises exception on failure
+    spec122_validator.validate(valid_spec_122_header_datainsecondHDU, return_type=Path)
